#include <stdlib.h>
#include <termios.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/time.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <signal.h>
#include <ctype.h>

/**********************************************************/
/*                                                        */
/*  openRaw                                               */
/*                                                        */
/*  Opens passed filename and sets its device to raw mode.*/
/*  Returns file descriptor.                              */
/*                                                        */
/*  raw mode is defined as:                               */
/*  Off: echo, canonical input, extended processing,      */
/*       signals, break key, parity, 8th bit strip,       */
/*       flow control, output post processing             */
/*   On: 8 bit size                                       */
/*                                                        */
/**********************************************************/

int openRaw(const char * filename, mode_t io_flags)
{
  struct termios term_info;
  int fd;

  fd = open(filename,io_flags);
  if (fd == -1) {
    /* maybe complain if TRACE is on */
    return 0;
  }

  if(tcgetattr(fd,&term_info) <0) {
    /* complain - fd is not a terminal */
    return 0;
  }

  /* turn off echo, canonical mode, extended processing, signals */
  term_info.c_lflag &= ~(ECHO | ICANON | IEXTEN | ISIG);

  /* turn off break sig, cr->nl, parity off, 8 bit strip, flow control */
  term_info.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP | IXON);

  /* clear size, turn off parity bit */
  term_info.c_cflag &= ~(CSIZE | PARENB);

  /* set size to 8 bits */
  term_info.c_cflag |= CS8;

  /* turn output processing off */
  term_info.c_oflag &= ~(OPOST);

  /* Set time and bytes to read at once */
  term_info.c_cc[VTIME] = 0;
  term_info.c_cc[VMIN] = 0;

  if(tcsetattr(fd,TCSAFLUSH,&term_info) <0)
    return -1;

  return fd;
}


/**********************************************************/
/*                                                        */
/*  setBaudRate                                           */
/*                                                        */
/*  Set the baud rate of an open device.                  */
/*                                                        */
/*  Baud rate constants are found in <termios.h>.         */
/*  B9600 == 9600 baud                                    */
/*                                                        */
/**********************************************************/

int setBaudRate (int fd, speed_t baudRate)
{
  struct termios terminfo;
  int error;

  error = tcgetattr(fd, &terminfo);
  if (error) {
    perror("tcgetattr()");
    return 1;
  }

  error = cfsetospeed(&terminfo, baudRate);
  if (error) {
    fprintf(stderr, "cfsetospeed(%ld): ",
            (long)baudRate);
    perror(NULL);
    return 1;
  }
 
  error = cfsetispeed(&terminfo, baudRate);
  if (error) {
    fprintf(stderr, "cfsetispeed(%ld): ",
            (long)baudRate);
    perror(NULL);
    return 1;
  }

  error = tcsetattr(fd, TCSANOW, &terminfo);
  if (error) {
    perror("tcsetattr()");
    return 1;
  }
  
  return 0;
}

int openSerialPort (char *fileName, int baudRate)
{
  int fd;
  speed_t baud;
  if (baudRate == 115200) baud= B115200;
  if (baudRate == 9600) baud= B9600;

  fd = openRaw(fileName, O_RDWR | O_NONBLOCK);
  if (fd<0) {
    fprintf(stderr, "unable to open port\n");
    exit(1);
  }
  if (setBaudRate(fd, baud)) {
    fprintf(stderr, "unable to set baud rate on port\n");
    exit(1);
  }
  return fd;
}
  
